;
; Driver TCP/IP
;
; docs: https://github.com/Konamiman/MSX-UNAPI-specification/blob/master/docs/TCP-IP%20UNAPI%20specification.md
;
;
;

;4.1.2. TCPIP_GET_CAPAB: Get information about the TCP/IP capabilities and features
;Input:  A  = 1
;        B  = Index of information block to retrieve:
;            1: Capabilities and features flags, link level protocol
;            2: Connection pool size and status
;            3: Maximum datagram size allowed
;Output: A = Error code
;        When information block 1 requested:
;            HL = Capabilities flags
;            DE = Features flags
;            B  = Link level protocol used
;        When information block 2 requested:
;            B  = Maximum simultaneous TCP connections supported
;            C  = Maximum simultaneous UDP connections supported
;            D  = Free TCP connections currently available
;            E  = Free UDP connections currently available
;            H  = Maximum simultaneous raw IP connections supported
;            L  = Free raw IP connections currently available
;        When information block 3 requested:
;            HL = Maximum incoming datagram size supported
;            DE = Maximum outgoing datagram size supported
TCPIP_GET_CAPAB:
	MIDI_TX_START 1
	MIDI_TX_B
	MIDI_TX_END
	MIDI_RX_START
	MIDI_RX_A_PUSH
	MIDI_RX_BC
	MIDI_RX_DE
	MIDI_RX_HL
	MIDI_RX_END_ERR_RET

;4.1.3. TCPIP_GET_IPINFO: Get IP address
;Input:  A  = 2
;        B  = Index of address to obtain:
;             1: Local IP address
;             2: Peer IP address
;             3: Subnet mask
;             4: Default gateway
;             5: Primary DNS server IP address
;             6: Secondary DNS server IP address
;Output: A  = Error code
;        L.H.E.D = Requested address
TCPIP_GET_IPINFO:
	MIDI_TX_START 2
	MIDI_TX_B
	MIDI_TX_END
	MIDI_RX_START
	MIDI_RX_A_PUSH
	MIDI_RX_DE
	MIDI_RX_HL
	MIDI_RX_END_ERR_RET

;4.1.4. TCPIP_NET_STATE: Get network state
;Input:  A  = 3
;Output: A  = Error code
;        B  = Current network state:
;             0: Closed
;             1: Opening
;             2: Open
;             3: Closing
;           255: Unknown
TCPIP_NET_STATE:
	MIDI_TX_START 3
	MIDI_TX_END
	MIDI_RX_START
	MIDI_RX_A_PUSH
	MIDI_RX_B
	MIDI_RX_END_ERR_RET

;4.2.1. TCPIP_SEND_ECHO: Send ICMP echo message (PING)
;Input:  A  = 4
;        HL = Address of echo parameters block
;Output: A = Error code
TCPIP_SEND_ECHO:
	MIDI_TX_START 4
	; 4 = ip addr
	LD		A,(HL)
	MIDI_TX_A
	INC		HL
	LD		A,(HL)
	MIDI_TX_A
	INC		HL
	LD		A,(HL)
	MIDI_TX_A
	INC		HL
	LD		A,(HL)
	MIDI_TX_A
	INC		HL
	; 1 = TTL
	LD		A,(HL)
	MIDI_TX_A
	INC		HL
	; 2 = icmp ident
	LD		A,(HL)
	MIDI_TX_A
	INC		HL
	LD		A,(HL)
	MIDI_TX_A
	INC		HL
	; 2 = icmp seq num
	LD		A,(HL)
	MIDI_TX_A
	INC		HL
	LD		A,(HL)
	MIDI_TX_A
	;INC		HL
	MIDI_TX_END
	MIDI_RX_START
	MIDI_RX_A_PUSH
	MIDI_RX_END_ERR_RET

;4.2.2. TCPIP_RCV_ECHO: Retrieve ICMP echo response message
;Input:  A  = 5
;        HL = Address for the echo parameters block
;Output: A = Error code
TCPIP_RCV_ECHO:
	MIDI_TX_START 5
	MIDI_TX_END
	MIDI_RX_START
	MIDI_RX_A_PUSH
	
	MIDI_RX_END_ERR_RET

;4.3.1. TCPIP_DNS_Q: Start a host name resolution query
;Input:  A  = 6
;        HL = Address of the host name to be resolved, zero terminated
;        B  = Flags, when set to 1 they instruct the resolver to:
;             bit 0: Only abort the query currently in progress, if there is any
;                    (other flags and registers are then ignored)
;             bit 1: Assume that the passed name is an IP address,
;                    and return an error if this is not true
;             bit 2: If there is a query in progress already,
;                    do NOT abort it and return an error instead
;Output: A  = Error code
;        B = 0 if a query to a DNS server is in progress
;            1 if the name represented an IP address
;            2 if the name could be resolved locally
;        L.H.E.D = Resolved IP address
;            (only if no error occurred and B=1 or 2 is returned)
TCPIP_DNS_Q:
	MIDI_TX_START 6
	MIDI_TX_HL
	MIDI_TX_B
	MIDI_TX_END
	MIDI_RX_START
	MIDI_RX_A_PUSH
	MIDI_RX_B
	MIDI_RX_DE
	MIDI_RX_HL
	MIDI_RX_END_ERR_RET

;4.3.2. TCPIP_DNS_S: Obtains the host name resolution process state and result
;Input:  A  = 7
;        B  = Flags, when set to 1 they instruct the resolver to:
;             bit 0: Clear any existing result or error condition after the execution
;                    (except if there is a query in progress)
;Output: A  = Error code
;        B  = DNS error code (when error is ERR_DNS)
;        B  = Current query status (when error is ERR_OK):
;             0: There is no query in progress, nor any result nor error code available
;             1: There is a query in progress
;             2: Query is complete
;        C  = Current query substatus (when error is ERR_OK and B=1):
;             0: Unknown
;             1: Querying the primary DNS server
;             2: Querying the secondary DNS server
;             3: Querying another DNS server
;        C  = Resolution proces type (when error is ERR_OK and B=2):
;             0: The name was obtained by querying a DNS server
;             1: The name was a direct representation of an IP address
;             2: The name was resolved locally
;   L.H.E.D = Resolved IP address (when error is ERR_OK and B=2)
TCPIP_DNS_S:
	ret_no_impl

;4.4.1. TCPIP_UDP_OPEN: Open a UDP connection
;Input:  A  = 8
;        HL = Local port number (0FFFFh: random port)
;        B  = Intended connection lifetime:
;             0: Transient
;             1: Resident
;Output: A  = Error code
;        B  = Connection number
TCPIP_UDP_OPEN:
	MIDI_TX_START 8
	MIDI_TX_HL
	MIDI_TX_B
	MIDI_TX_END
	MIDI_RX_START #TCPIP_UDP_OPEN
	MIDI_RX_A_PUSH
	MIDI_RX_B
	MIDI_RX_END_POP
	RET

;4.4.2. TCPIP_UDP_CLOSE: Close a UDP connection
;Input:  A  = 9
;        B  = Connection number
;             0 to close all open transient UDP connections
;Output: A  = Error code
TCPIP_UDP_CLOSE:
	ret_no_impl

;4.4.3. TCPIP_UDP_STATE: Get the state of a UDP connection
;Input:  A  = 10
;        B  = Connection number
;Output: A  = Error code
;        HL = Local port number
;        B  = Number of pending incoming datagrams
;        DE = Size of oldest pending incoming datagram (data part only)
TCPIP_UDP_STATE:
	ret_no_impl

;4.4.4. TCPIP_UDP_SEND: Send an UDP datagram
;Input:  A  = 11
;        B  = Connection number
;        HL = Address of datagram data
;        DE = Address of parameters block
;Output: A  = Error code
TCPIP_UDP_SEND:
	ret_no_impl

;4.4.5. TCPIP_UDP_RCV: Retrieve an incoming UDP datagram
;Input:  A  = 12
;        B  = Connection number
;        HL = Address for datagram data
;        DE = Maximum data size to retrieve
;Output: A  = Error code
;   L.H.E.D = Source IP address
;        IX = Source port
;        BC = Actual received data size
TCPIP_UDP_RCV:
	ret_no_impl

;4.5.1. TCPIP_TCP_OPEN: Open a TCP connection
;Input:  A  = 13
;        HL = Address of parameters block
;Output: A  = Error code
;        B  = Connection number
TCPIP_TCP_OPEN:
	ret_no_impl

;4.5.2. TCPIP_TCP_CLOSE: Close a TCP connection
;Input:  A  = 14
;        B  = Connection number
;             0 to close all open transient TCP connections
;Output: A  = Error code
TCPIP_TCP_CLOSE:
	ret_no_impl

;4.5.3. TCPIP_TCP_ABORT: Abort a TCP connection
;Input:  A  = 15
;        B  = Connection number
;             0 to abort all open transient TCP connections
;Output: A  = Error code
TCPIP_TCP_ABORT:
	ret_no_impl

;4.5.4. TCPIP_TCP_STATE: Get the state of a TCP connection
;Input:  A  = 16
;        B  = Connection number
;        HL = Pointer in TPA for connection information block
;             (0 if not needed)
;Output: A  = Error code
;        B  = Connection state
;        C  = Close reason (only if ERR_NO_CONN is returned)
;        HL = Number of total available incoming bytes
;        DE = Number of urgent available incoming bytes
;        IX = Available free space in the output buffer
;             (0FFFFh = infinite)
TCPIP_TCP_STATE:
	ret_no_impl

;4.5.5. TCPIP_TCP_SEND: Send data a TCP connection
;Input:  A  = 17
;        B  = Connection number
;        DE = Address of the data to be sent
;        HL = Length of the data to be sent
;        C  = Flags:
;             bit 0: Send the data PUSHed
;             bit 1: The data is urgent
;Output: A  = Error code
TCPIP_TCP_SEND:
	ret_no_impl

;4.5.6. TCPIP_TCP_RCV: Receive data from a TCP connection
;Input:  A  = 18
;        B  = Connection number
;        DE = Address for the retrieved data
;        HL = Length of the data to be obtained
;Output: A  = Error code
;        BC = Total number of bytes that have been actually retrieved
;        HL = Number of urgent data bytes that have been retrieved
;             (placed at the beginning of the received data block)
TCPIP_TCP_RCV:
	ret_no_impl


;4.5.7. TCPIP_TCP_FLUSH: Flush the output buffer of a TCP connection
;Input:  A  = 19
;        B  = Connection number
;Output: A  = Error code
TCPIP_TCP_FLUSH:
	ret_no_impl

;4.6.1. TCPIP_RAW_OPEN: Open a raw IP connection
;Input:  A  = 20
;        B  = Transport protocol code
;        C  = Intended connection lifetime:
;             0: Transient
;             1: Resident
;Output: A  = Error code
;        B  = Connection number
TCPIP_RAW_OPEN:
	ret_no_impl

;4.6.2. TCPIP_RAW_CLOSE: Close a raw IP connection
;Input:  A  = 21
;        B  = Connection number
;             0 to close all open transient UDP connections
;Output: A  = Error code
TCPIP_RAW_CLOSE:
	ret_no_impl

;4.6.3. TCPIP_RAW_STATE: Get the state of a raw IP connection
;Input:  A  = 22
;        B  = Connection number
;Output: A  = Error code
;        B  = Associated protocol code
;        HL = Number of pending incoming datagrams
;        DE = Size of the oldest pending incoming datagram
TCPIP_RAW_STATE:
	ret_no_impl

;4.6.4. TCPIP_RAW_SEND: Send a raw IP datagram
;Input:  A  = 23
;        B  = Connection number
;        HL = Address of datagram data
;        DE = Address of parameters block
;Output: A  = Error code
TCPIP_RAW_SEND:
	ret_no_impl

;4.6.5. TCPIP_RAW_RCV: Retrieve an incoming raw IP datagram
;Input:  A  = 24
;        B  = Connection number
;        HL = Address for datagram data
;        DE = Maximum data size to retrieve
;Output: A  = Error code
;   L.H.E.D = Source IP address
;        BC = Actual received data size
TCPIP_RAW_RCV:
	ret_no_impl

;4.7.1. TCPIP_CONFIG_AUTOIP: Enable or disable the automatic IP addresses retrieval
;Input:  A  = 25
;        B  = 0: Get current configuration
;             1: Set configuration
;        C  = Configuration to set (only if B=1):
;             bit 0: Set to automatically retrieve
;                    local IP address, subnet mask and default gateway
;             bit 1: Set to automatically retrieve DNS servers addresses
;             bits 2-7: Unused, must be zero
;Output: A  = Error code
;        C  = Configuration after the routine execution
;             (same format as C at input)
TCPIP_CONFIG_AUTOIP:
	ret_no_impl

;4.7.2. TCPIP_CONFIG_IP: Manually configure an IP address
;Input:  A  = 26
;        B  = Index of address to set:
;             1: Local IP address
;             2: Peer IP address
;             3: Subnet mask
;             4: Default gateway
;             5: Primary DNS server IP address
;             6: Secondary DNS server IP address
;   L.H.E.D = Address value
;Output: A  = Error code
TCPIP_CONFIG_IP:
	ret_no_impl

;4.7.3. TCPIP_CONFIG_TTL: Get/set the value of TTL and TOS for outgoing datagrams
;Input:  A  = 27
;        B  = 0: Get current values
;             1: Set values
;        D  = New value for TTL (only if B=1)
;        E  = New value for ToS (only if B=1)
;Output: A  = Error code
;        D  = Value of TTL after the routine execution
;        E  = Value of ToS after the routine execution
TCPIP_CONFIG_TTL:
	ret_no_impl

;4.7.4. TCPIP_CONFIG_PING: Get/set the automatic PING reply flag
;Input:  A  = 28
;        B  = 0: Get current flag value
;             1: Set flag value
;        C  = New flag value (only if B=1):
;             0: Off
;             1: On
;Output: A  = Error code
;        C  = Flag value after the routine execution
TCPIP_CONFIG_PING:
	ret_no_impl

;4.8.1. TCPIP_WAIT: Wait for a processing step to run
;Input:  A  = 29
;Output: A  = Error code
TCPIP_WAIT:
	ret_no_impl

