;
;
; 0x4000 - extbio unapi
; 0x5000 -
; 0x6000 -
; 0x7000 -
;

ARG		.equ	0xF847

;--- API version and implementation version

API_V_P		.equ	1
API_V_S		.equ	0
ROM_V_P		.equ	1
ROM_V_S		.equ	1

;--- Maximum number of available standard and implementation-specific function numbers

;Must be 0 to 127
MAX_FN		.equ	30+1

;Must be either zero (if no implementation-specific functions available), or #128 to 254
MAX_IMPFN	.equ	127+3


;*********************************************
;***  CODE TO BE INSTALLED ON RAM SEGMENT  ***
;*********************************************

	;===============================
	;===  EXTBIO hook execution  ===
	;===============================

	;>>> Note that this code starts exactly at address 0x4000

;	.ds 129
;.area CODE1 (ABS)
;	 .org 0H4400

DO_EXTBIO:
	push	hl
	push	bc
	push	af
	ld	a,d
	cp	#0x22
	jr	nz,JUMP_OLD
	cp	e
	jr	nz,JUMP_OLD

	;Check API ID

	ld	hl,#UNAPI_ID
	ld	de,#ARG
LOOP:	ld	a,(de)
	call	TOUPPER
	cp	(hl)
	jr	nz,JUMP_OLD2
	inc	hl
	inc	de
	or	a
	jr	nz,LOOP

	;A=255: Jump to old hook
	pop	af
	push	af
	inc	a
	jr	z,JUMP_OLD2

	;A=0: B=B+1 and jump to old hook
	pop	af
	pop	bc
	or	a
	jr	nz,DO_EXTBIO2
	inc	b
	pop	hl
	ld	de,#0x2222
	jp	OLD_EXTBIO

DO_EXTBIO2:
	;A=1: Return A=Slot, B=Segment, HL=UNAPI entry address
	dec	a
	jr	nz,DO_EXTBIO3
	pop	hl
	ld	a,(MY_SEG)
	ld	b,a
	ld	a,(MY_SLOT)
	ld	hl,#UNAPI_ENTRY
	ld	de,#0x2222
	ret

	;A>1: A=A-1, and jump to old hook

DO_EXTBIO3:	;A=A-1 already done
	pop	hl
	ld	de,#0x2222
	jp	OLD_EXTBIO


	;--- Jump here to execute old EXTBIO code

JUMP_OLD2:
	ld	de,#0x2222
JUMP_OLD:	;Assumes "push hl,bc,af" done
	pop	af
	pop	bc
	pop	hl

	;Old EXTBIO hook contents is here
	;(it is setup at installation time)

OLD_EXTBIO:
	.ds	5


	;====================================
	;===  Functions entry point code  ===
	;====================================

UNAPI_ENTRY:
	push	hl
	push	af
	ld	hl,#FN_TABLE
	bit	7,a
	jr	z,IS_STANDARD
	ld	hl,#IMPFN_TABLE
	and	#0b01111111
	cp	#MAX_IMPFN-128
	jr	z,OK_FNUM
	jr	nc,UNDEFINED
IS_STANDARD:
	cp	#MAX_FN
	jr	z,OK_FNUM
	jr	nc,UNDEFINED
OK_FNUM:
	add	a,a
	push	de
	ld	e,a
	ld	d,#0
	add	hl,de
	pop	de

	ld	a,(hl)
	inc	hl
	ld	h,(hl)
	ld	l,a

	pop	af
	ex	(sp),hl
	ret

	;--- Undefined function: return with registers unmodified

UNDEFINED:
	pop	af
	pop	hl
	ret

;--- Convert a character to upper-case if it is a lower-case letter

TOUPPER:
	cp	#"a"
	ret	c
	cp	#"z"+1
	ret	nc
	and	#0xDF
	ret

	;===================================
	;===  Functions addresses table  ===
	;===================================

;--- Standard routines addresses table

FN_TABLE:
	.dw	UNAPI_GET_INFO
	.dw	TCPIP_GET_CAPAB
	.dw	TCPIP_GET_IPINFO
	.dw	TCPIP_NET_STATE
	.dw	TCPIP_SEND_ECHO
	.dw	TCPIP_RCV_ECHO
	.dw	TCPIP_DNS_Q
	.dw	TCPIP_DNS_S
	.dw	TCPIP_UDP_OPEN
	.dw	TCPIP_UDP_CLOSE
	.dw	TCPIP_UDP_STATE
	.dw	TCPIP_UDP_SEND
	.dw	TCPIP_UDP_RCV
	.dw	TCPIP_TCP_OPEN
	.dw	TCPIP_TCP_CLOSE
	.dw	TCPIP_TCP_ABORT
	.dw	TCPIP_TCP_STATE
	.dw	TCPIP_TCP_SEND
	.dw	TCPIP_TCP_RCV
	.dw	TCPIP_TCP_FLUSH
	.dw	TCPIP_RAW_OPEN
	.dw	TCPIP_RAW_CLOSE
	.dw	TCPIP_RAW_STATE
	.dw	TCPIP_RAW_SEND
	.dw	TCPIP_RAW_RCV
	.dw	TCPIP_CONFIG_AUTOIP
	.dw	TCPIP_CONFIG_IP
	.dw	TCPIP_CONFIG_TTL
	.dw	TCPIP_CONFIG_PING
	.dw	TCPIP_WAIT
	.dw	0


;--- Implementation-specific routines addresses table

IMPFN_TABLE:
	.dw UNAPIE_INIT
	.dw	UNAPIE_GET_JUMP
	.dw	DRV_INIT
	.dw	DRV_GET_INFO
	;.dw	TCPIPE_STATS
	;.dw	TCPIPE_RESET
	.dw	0



;UNAPI_GET_INFO: Obtain the implementation name and version
;Input:  A  = 0
;Output: A  = Error code
;        HL = Address of the implementation name string
;        DE = API specification version supported. D=primary, E=secondary.
;        BC = API implementation version. B=primary, C=secondary.
UNAPI_GET_INFO:
	LD		BC,#256*ROM_V_P+ROM_V_S
	LD		DE,#256*API_V_P+API_V_S
	LD		HL,#APIINFO
	XOR		A
	RET


;UNAPIE_INIT: Inits unapi jump code
;Input: A  = 128
;       HL = slot and segment ids
;       BC = copy 5 bytes from this addres to OLD_EXTBIO
;Output:
;		A  = Error code
;
;The routines of the XXXX specification are intended to be called intensively, therefore being performance a concern. For this reason, a jump table is provided so that all the routines can be invoked directly in addition to using the standard UNAPI entry point. Routines are ordered by their UNAPI routine number, so that routine 0 is at (HL), routine 1 is at (HL+3), and so on.
UNAPIE_INIT:
	LD		(MY_SLOT),HL
	LD		H,B
	LD		L,C
	LD		DE,#OLD_EXTBIO
	LD		BC,#5
	LDIR
	RET

;UNAPIE_GET_JUMP: Get the routines jump table address
;Input: A = 129
;Output: HL = Address of the jump table (possibly in page 2)
;A = Segment of the jump table (#FF if ROM)
;
;The routines of the XXXX specification are intended to be called intensively, therefore being performance a concern. For this reason, a jump table is provided so that all the routines can be invoked directly in addition to using the standard UNAPI entry point. Routines are ordered by their UNAPI routine number, so that routine 0 is at (HL), routine 1 is at (HL+3), and so on.
UNAPIE_GET_JUMP:
	LD		HL,#FN_TABLE
	LD		A,(#MY_SEG)
	RET


;DRV_INIT: Inits the implementation hardware.
;Input:  A  = 130
;Output: A  = Error code
;        HL = Address of the implementation name string
;        DE = API specification version supported. D=primary, E=secondary.
;        BC = API implementation version. B=primary, C=secondary.
DRV_INIT:
	MIDI_INIT
	XOR		A
	RET

;DRV_GET_INFO: Obtain the implementation gateway name and version
;Input:  A  = 131
;Output: A  = Error code
;        HL = Address of the implementation hostname string
;        DE = Connected gateway version. D=primary, E=secondary.
DRV_GET_INFO:
	MIDI_TX_START 131
	MIDI_TX_END
	MIDI_RX_START #DRV_GET_INFO
	MIDI_RX_A_PUSH
	;MIDI_RX_STR #DRV_HOSTNAME
	;MIDI_RX_DE
	MIDI_RX_END_POP
	LD	HL,#DRV_HOSTNAME
	RET

DRV_HOSTNAME:
		.str "hostname"
		.str	"$"
		.ds 40

	;============================
	;===  UNAPI related data  ===
	;============================

;This data is setup at installation time

MY_SLOT:	.db	0
MY_SEG:		.db	0


	;--- Specification identifier (up to 15 chars)

UNAPI_ID:
		.str	"TCP/IP"
		.db		0
UNAPI_ID_END:

	;--- Implementation name (up to 63 chars and zero terminated)

APIINFO:
		.str	"mmidrv-rpc"
		.str	"$"
		.db		0

